"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var api_load_resolve_1 = require("./api-load-resolve");
var api_page_component_1 = require("./api-page.component");
var _ = require("lodash");
function createApiRoutes(def) {
    var routes = [];
    if (def['versions'] && def['files']) {
        var apiDefinition_1 = def;
        apiDefinition_1.versions.forEach(function (apiVersion) {
            var apiVersionName = apiVersion.name;
            apiDefinition_1.files.forEach(function (apiFile) {
                var fileName = apiFile.apiTitle;
                var apiUrl = apiVersionName + '/' + fileName;
                apiVersion.exclude = apiVersion.exclude || [];
                if (apiVersion.exclude && apiVersion.exclude.indexOf(fileName) === -1) {
                    routes.push({
                        path: apiUrl,
                        component: api_page_component_1.ApiPageComponent,
                        resolve: {
                            resolveData: api_load_resolve_1.ApiLoadResolve
                        },
                        data: {
                            isStaticApi: false,
                            apiDefinition: apiDefinition_1,
                            apiVersion: apiVersion,
                            apiFile: apiFile
                        }
                    });
                }
            });
        });
    }
    else if (def['apis']) {
        var apiDefinition_2 = def;
        apiDefinition_2.apis.forEach(function (apiItem) {
            var apiVersion = apiItem.version;
            if (apiItem.ref && apiItem.ref !== '') {
                var refApiItem = apiDefinition_2.apis.find(function (item) {
                    return item.version === apiItem.ref;
                });
                apiItem.files = refApiItem.files;
            }
            if (apiItem.alongWith && apiItem.alongWith !== '') {
                var alongApiItem = apiDefinition_2.apis.find(function (item) {
                    return item.version === apiItem.alongWith;
                });
                var withoutDups = _.filter(alongApiItem.files, function (item) {
                    var found = apiItem.files.find(function (existsItem) {
                        return existsItem.name === item.name;
                    });
                    return !found;
                });
                apiItem.files = apiItem.files.concat(withoutDups);
            }
            var redirectionUrl = '/';
            apiItem.files.forEach(function (apiFile, index) {
                var apiUrl = apiVersion + '/' + apiFile.urlName;
                if (index === 0) {
                    redirectionUrl = apiUrl;
                }
                routes.push({
                    path: apiUrl,
                    component: api_page_component_1.ApiPageComponent,
                    resolve: {
                        resolveData: api_load_resolve_1.ApiLoadResolve
                    },
                    data: {
                        isStaticApi: true,
                        apiDefinition: apiDefinition_2,
                        apiVersion: apiItem,
                        apiFile: apiFile
                    }
                });
            });
            routes.push({
                path: apiVersion,
                redirectTo: redirectionUrl
            });
        });
    }
    return routes;
}
exports.createApiRoutes = createApiRoutes;
//# sourceMappingURL=apis-routes.js.map