"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require('fs');
var request = require('request');
var mkdirp = require('mkdirp');
var path = require('path');
var rimraf = require('rimraf');
var copydir = require('copy-dir');
var moment = require('moment');
function ensureDirectoryExistence(filePath) {
    var dirname = path.dirname(filePath);
    if (fs.existsSync(dirname)) {
        return true;
    }
    ensureDirectoryExistence(dirname);
    fs.mkdirSync(dirname);
}
exports.handleRoutesArray = function (arr, resultArr, base) {
    if (base === void 0) { base = '/'; }
    arr.forEach(function (route) {
        if (route.path !== '**') {
            resultArr.push(base + route.path);
            if (route.children && route.children.length > 0) {
                exports.handleRoutesArray(route.children, resultArr, base + route.path + '/');
            }
        }
    });
};
function generateStaticWebsite(baseHost, port, routesArray, outputLocation, assetsDirectory) {
    var siteMapArr = [];
    function generateSiteMap(pathArr) {
        console.log('Generating sitemap.xml file');
        return new Promise(function (resolve) {
            var base = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><urlset xmlns=\"http://www.sitemaps.org/schemas/sitemap/0.9\">";
            pathArr.forEach(function (p) {
                var path = p;
                if (path === 'index') {
                    path = '/';
                }
                var priority;
                var depth = (path.split('/') || []).length;
                if (depth === 0 || depth === 1) {
                    priority = '1.0';
                }
                else if (depth === 2) {
                    priority = '0.8';
                }
                else {
                    priority = '0.5';
                }
                base += "\n    <url>\n        <loc>http://angular-meteor.com" + path + "</loc>\n        <lastmod>" + moment().format('YYYY-MM-DD') + "</lastmod>\n        <changefreq>weekly</changefreq>\n        <priority>" + priority + "</priority>\n    </url>";
            });
            base += "</urlset>";
            var fullFilePath = path.join(outputLocation, 'sitemap.xml');
            fs.writeFileSync(fullFilePath, base);
            resolve();
        });
    }
    function generateStaticForUrl(fullBaseUrl, urlPath) {
        var url = fullBaseUrl + urlPath;
        return new Promise(function (resolve, reject) {
            request(url, function (err, res) {
                if (err) {
                    return reject(err);
                }
                console.log('Done loading HTML content for ' + url);
                resolve({
                    html: res.body,
                    url: urlPath
                });
            });
        });
    }
    rimraf.sync(outputLocation);
    copydir.sync(assetsDirectory, path.join(outputLocation, path.basename(assetsDirectory)), function (stat, filepath, filename) {
        if (stat === 'file' && (path.extname(filepath) === '.md' || path.extname(filepath) === '.patch')) {
            return false;
        }
        return true;
    });
    var fullBaseUrl = 'http://' + baseHost + ':' + port;
    var urlsToLoad = ['/'];
    exports.handleRoutesArray(routesArray, urlsToLoad);
    var r;
    var donePromise = new Promise(function (resolve) {
        r = resolve;
    });
    function runNext(arr, i) {
        if (i === void 0) { i = 0; }
        if (arr[i]) {
            generateStaticForUrl(fullBaseUrl, arr[i]).then(function (result) {
                var filePath = result.url;
                if (filePath === '/') {
                    filePath = 'index';
                }
                siteMapArr.push(filePath);
                var filePathIndex = filePath + '/index.html';
                filePath = filePath + '.html';
                var fullFilePath = path.join(outputLocation, filePath);
                var fullFilePathIndex = path.join(outputLocation, filePathIndex);
                mkdirp.sync(path.dirname(fullFilePath));
                mkdirp.sync(path.dirname(fullFilePathIndex));
                fs.writeFileSync(fullFilePath, result.html);
                fs.writeFileSync(fullFilePathIndex, result.html);
                console.log('Done writing to files:' + [fullFilePath, fullFilePathIndex].join(', '));
                runNext(arr, i + 1);
            });
        }
        else {
            generateSiteMap(siteMapArr).then(function () {
                r();
            });
        }
        return donePromise;
    }
    return runNext(urlsToLoad);
}
exports.generateStaticWebsite = generateStaticWebsite;
//# sourceMappingURL=generate-static.js.map